/*
 * math.c
 *
 * Created: 24.5.2013 18:24:36
 *  Author: xkralj07
 */

#define F_CPU					1000000
#include <stdint.h>
#include <avr/io.h>
#include <util/delay.h>

#include "lcd_driver.h"

void LcdSendByte(uint8_t byte)
{
	LcdDataBusSet(byte >> 4);					// upper nibble first
	LCD_INSTR_ENABLE();
	LCD_INSTR_DISABLE();

	LcdDataBusSet(byte);						// lower nibble
	LCD_INSTR_ENABLE();
	LCD_INSTR_DISABLE();
}

void LcdTurnOn(void)
{
	LcdPowerOn();								// power on LCD display
	
	// wait 15 ms for display ends initialization
	_delay_ms(16);
	
	LCD_INSTR_CMD();
	
	// set 4-bits interface
	LcdDataBusSet(0x20 >> 4);					// upper nibble first
	LCD_INSTR_ENABLE();
	LCD_INSTR_DISABLE();
	_delay_us(45);
	
	LcdSendByte(0x28);							// set two lines of display
	_delay_us(45);
	LcdSendByte(0x04);							// set cursor auto-shift
	_delay_us(45);
	LcdSendByte(0x0c);							// turn display on	
	_delay_us(45);
	
	LcdSendByte(0x01);							// turn display on
	_delay_us(1600);
}

void LcdPrintString(char *str)
{
	LCD_INSTR_DATA();

	while(*str != '\0')							// send all characters from the string
	{
		LcdSendByte(*str);						// send character from string to LCD display
		_delay_us(45);							// delay for instruction could be processed by LCD display
		
		str ++;									// increment string pointer
	}
}

void LcdPrintCharDelayed(char c)
{
	LCD_INSTR_DATA();
	LcdSendByte(c);
	_delay_us(45);
}

void LcdPrintCharNoDelay(char c)
{
	LCD_INSTR_DATA();
	LcdSendByte(c);
}

void LcdPrintInteger(uint8_t num)
{
	char num_str [4] = "   ";
	char *str;

	str = &num_str[2];
	do
	{
		*str = num % 10 + '0';					// remainder after integer division converted to character
		num = num / 10;							// divide integer part by 10
		str --;									// decrement string pointer - it is safe because 8-bit number has not to be greater than 256, so less than 3 decimal places
	}while(num != 0);

	LcdPrintString(str);
}

void LcdPrintFixedPoint(uint16_t num, uint8_t decim_point)
{
	uint8_t num_temp = num >> decim_point;
	uint8_t num_int = 0;

	LcdPrintInteger(num_temp);

	num_temp = num ^ ((uint16_t)(num_temp) << decim_point);			// clear upper bits that corresponds to integer of number
	
	// convert fractional part to integer
	for (uint8_t i = 0; i < decim_point; i ++)
	{
		num_int = num_int >> 1;
		
		if (num_temp & 0x01)					// if LSB of num_temp is set
			num_int += 50;
			
		num_temp = num_temp >> 1;
	}
	
	LcdPrintCharDelayed('.');
	LcdPrintInteger(num_int);
}