/*
 * lcd_driver.h
 *
 * Created: 24.5.2013 18:42:00
 *  Author: xkralj07
 */ 


#ifndef LCD_DRIVER_H_
#define LCD_DRIVER_H_

#include <avr/io.h>

#define LcdDataBusSet(x)		(PORTB  = (PORTB & 0xf0) | (x & 0x0f))
#define LcdDataBusInit()		(DDRB  |=  0x0f)							// whole lower half of port B is data bus outputs

#define LCD_DATA_BUS_HI_Z()		\
do {							\
	PORTB &= ~0x0f;				\
	DDRB  &= ~0x0f;				\
} while (0)


#define LCD_INSTR_READ()		PORTD |=  _BV(PIND6)
#define LCD_INSTR_WRITE()		PORTD &= ~_BV(PIND6)
#define LCD_INSTR_CMD()			PORTD &= ~_BV(PIND7)
#define LCD_INSTR_DATA()		PORTD |=  _BV(PIND7)
#define LCD_INSTR_ENABLE()		PORTD |=  _BV(PIND5)
#define LCD_INSTR_DISABLE()		PORTD &= ~_BV(PIND5)
#define LCD_INSTR_INIT()		DDRD  |=  _BV(PIND5) | _BV(PIND6) | _BV(PIND7)

#define LCD_INSTR_HI_Z()								\
do {													\
	PORTD &=~(_BV(PIND5) | _BV(PIND6) | _BV(PIND7));	\
	DDRD  &=~(_BV(PIND5) | _BV(PIND6) | _BV(PIND7));	\
} while (0)

#define LCD_BACKLIGHT_INIT()	(DDRB  |=  _BV(PINB6))
#define LcdBacklightOn()		(PORTB |=  _BV(PINB6))
#define LcdBacklightOff()		(PORTB &= ~_BV(PINB6))

// LCD_VSS is connected to PORTB 7
#define LcdPowerOn()			\
do {							\
	LcdDataBusInit();			\
	LCD_INSTR_INIT();			\
	LCD_INSTR_WRITE();			\
	PORTB &= ~_BV(PINB7);		\
} while (0)

#define LcdPowerOff()			\
do {							\
	LcdBacklightOff();			\
	LCD_DATA_BUS_HI_Z();		\
	LCD_INSTR_HI_Z();			\
	PORTB |= _BV(PINB7);		\
} while (0)

#define LcdPowerInit()			\
do {							\
	LcdPowerOff();				\
	DDRB |= _BV(PINB7);			\
} while (0)

#define LcdInit()				\
do {							\
	LcdPowerInit();				\
	LCD_BACKLIGHT_INIT();		\
} while (0)


void LcdTurnOn(void);
void LcdPrintString(char *str);

void LcdPrintCharDelayed(char c);
void LcdPrintCharNoDelay(char c);

void LcdPrintInteger(uint8_t num);
void LcdPrintFixedPoint(uint16_t num, uint8_t decim_point);


#endif /* LCD_DRIVER_H_ */